package X11::Xlib::XRectangle;
require X11::Xlib::Struct;
__END__

=head1 NAME

X11::Xlib::XRectangle - Struct defining 16-bit x,y,width,height

=head1 ATTRIBUTES

=head2 x

16-bit signed

=head2 y

16-bit signed

=head2 width

16-bit unsigned

=head2 height

16-bit unsigned

=head1 METHODS

See parent class L<X11::Xlib::Struct>

=head1 AUTHOR

Olivier Thauvin, E<lt>nanardon@nanardon.zarb.orgE<gt>

Michael Conrad, E<lt>mike@nrdvana.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2009-2010 by Olivier Thauvin

Copyright (C) 2017 by Michael Conrad

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.10.0 or,
at your option, any later version of Perl 5 you may have available.

=cut
